"""series skin-effect resistance"""

# Copyright (c) 2018 Teledyne LeCroy, Inc.
# All rights reserved worldwide.
#
# This file is part of SignalIntegrity.
#
# SignalIntegrity is free software: You can redistribute it and/or modify it under the terms
# of the GNU General Public License as published by the Free Software Foundation, either
# version 3 of the License, or any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program.
# If not, see <https://www.gnu.org/licenses/>

from SignalIntegrity.Lib.Devices.SeriesZ import SeriesZ
from numpy import math

def SeriesRse(f,Rse,Z0=None):
    """Series Skin-effect Resistance
    @param Rse float resistance specified as ohms/sqrt(Hz)
    @param f float frequency
    @param Z0 (optional) float of complex reference impedance (defaults to 50 ohms)
    @return the list of list s-parameter matrix for a series resistance due to skin-effect
    """
    return SeriesZ(Rse*(1+1j)*math.sqrt(f),Z0)
