"""
VoltageControlledCurrentSource.py
"""

# Copyright (c) 2018 Teledyne LeCroy, Inc.
# All rights reserved worldwide.
#
# This file is part of SignalIntegrity.
#
# SignalIntegrity is free software: You can redistribute it and/or modify it under the terms
# of the GNU General Public License as published by the Free Software Foundation, either
# version 3 of the License, or any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program.
# If not, see <https://www.gnu.org/licenses/>

def VoltageControlledCurrentSource(G,Z0=50):
    """VoltageControlledCurrentSource
    Voltage controlled current source
    @param G float or complex transconductance
    @param Z0 (optional) float or complex reference impedance Z0 (defaults to 50 ohms)
    @return list of list s-parameter matrix of a voltage controlled current source
    @remark
    The four port voltage controlled current source has
    the voltage sense element sensing the voltage difference between the plus terminal
    at port 1 and the minus terminal at port 2.\n
    The current generated by the amplifier enters port 4 and exits port 3
    """
    return  [[1.,0.,0.,0.],
            [0.,1.,0.,0.],
            [2.*G*Z0,-2.*G*Z0,1.,0.],
            [-2.*G*Z0,2.*G*Z0,0.,1.]]